﻿using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Collections.Generic;
using LxS_Library;

namespace LxS_Recorder
{
    /// <summary>
    /// A simple Data-Recorder using LxS-Lib.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Simple List for Data-Storage
        /// </summary>
        static List<LxS_EventArgs> data_list = new List<LxS_EventArgs>();

        /// <summary>
        /// Data-Handler for Data comming from Sensor
        /// </summary>
        /// <param name="sender">LxS-Device raised the Event</param>
        /// <param name="e">Event with all Datas</param>
        static void lxs_DataHandler(object sender, LxS_EventArgs e)
        {
            // All incoming Datas will be stored to the List
            data_list.Add(e);
        }

        /// <summary>
        /// Sample Program to represent using of LxS_Library for
        /// Connect -> Measure(5s) -> Disconnect handling
        /// </summary>
        /// <param name="args"></param>
        static void Main(String[] args)
        {
            System.Console.Out.Write("LxS_Recorder\n1. Connecting to Sensor ");

            // Create new instance for a LxS-Device
            // Default: PC-Port 5634
            LxS lxs = new LxS_Device(5634);
            // Configure instance for kommunikation with sensor
            // Default: 192.168.60.3:9008
            lxs.DeviceIP = new IPEndPoint(IPAddress.Parse("192.168.60.3"), 9008);
            // Set 100ms for Timeout
            lxs.TimeOut = 100;

            // Establish a connection between instance and sensor
            while (!lxs.Connect())
            {
                System.Console.Out.Write(".");
                lxs.Disconnect();
                Thread.Sleep(100);  // Warning: Deadlock without sensor
            }

            System.Console.Out.WriteLine("\n2. Configuration");

            // Prepare sensor for messure true the connected instance:
            // 1. Setting sensor to COMMAND-Mode
            lxs.SetSensorMode(LxS_Mode.COMMAND);
            // 2. Prepare Event-Handler for Device-Datas
            lxs.DataHandler += new LxS_Event(lxs_DataHandler);
            // 3. Set Scan-Number and Encoder-Value to Zero
            lxs.ScanNumber = 0; lxs.EncoderValue = 0;
            // 4. Create instance of Device-Data-List for storing
            data_list = new List<LxS_EventArgs>();

            System.Console.Out.Write("3. Measuring ");

            // Starting measure for 5 seconds
            lxs.SetSensorMode(LxS_Mode.MEASURE);
            for (int i = 0; i < 5; i++)
            {
                Thread.Sleep(1000);
                System.Console.Out.Write(".");
            }
            lxs.SetSensorMode(LxS_Mode.COMMAND);

            System.Console.Out.WriteLine("\n4. Disconnect from Sensor");

            // Disconnect from sensor
            lxs.Disconnect();
            // Dispose instance and free all ressources
            lxs.Dispose();

            System.Console.Out.WriteLine("5. Write Datas to File: out.cvs");

            // Write stored data in to a CSV-File (overwrite old File)
            StreamWriter swr = new StreamWriter(File.Create("out.csv"));
            swr.WriteLine("ScnNr;EncVal;Cmd;DATAs;");
            foreach (LxS_EventArgs e in data_list)
            {
                // Converting UInt16 to signed Int16 only for X-Data-Package
                Boolean convert = e.Command.Equals(LxS_Cmd.X_COORDINATES);
                String line = e.ScanNumber.ToString() + ";";
                line += e.EncoderValue.ToString() + ";";
                line += e.Command.ToString() + ";";
                foreach (UInt16 p in e.Data)
                {
                    line += (convert ? ((Int16)p).ToString() : p.ToString()) + ";";
                }
                swr.WriteLine(line);
            }
            swr.Flush(); swr.Close();

            // The End
            System.Console.Out.WriteLine("Done, press ENTER for Quit");
            System.Console.In.Read();
        }
    }
}
