﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Net;
using System.Linq;
using System.Text;

using LxS_Library;
using LxS_VBA_Wrapper;




namespace LxS_VBA_Wrapper
{
    public interface LxS_Wrapper
    {
        bool Connect(string lps_ip, int lps_port, int pc_port);

        void Disconnect();

        void setLaserGate(bool state);

        void setSensorMode(bool mode);

        int getInspectionTask();

        void setInspectionTask(int task);

        string getVersion();

        string getData();

        string getField();

        string getDeviceType();

        void ethernetTrigger();
    }

    public class LxS_Device_Wrapper : LxS_Wrapper
    {
        private LxS lxs = null;
        private LxS_EventArgs last_z_data = null;
        private LxS_EventArgs old_z_data = null;
        private LxS_EventArgs last_x_data = null;
        private LxS_EventArgs last_field_data = null;

        public LxS_Device_Wrapper()
        {
            this.lxs = null;
            this.last_x_data = null;
            this.last_z_data = null;
            this.last_field_data = null;
        }

        #region LxS_Wrapper Member

        public bool Connect(string lps_ip, int lps_port, int pc_port)
        {
            bool ret = false;
            try
            {
                this.Disconnect();
                this.lxs = new LxS_Device(pc_port);
                this.lxs.DeviceIP = new IPEndPoint(IPAddress.Parse(lps_ip), lps_port);
                this.lxs.TimeOut = 500;
                ret = this.lxs.Connect();
                this.lxs.DataHandler += new LxS_Event(lxs_DataHandler);
            }
            catch
            {
                ret = false;
            }
            return (ret);
        }

        private void lxs_DataHandler(object sender, LxS_EventArgs e)
        {
            if ((e.Command == LxS_Cmd.ACK_Z_COORDINATES) || (e.Command == LxS_Cmd.Z_COORDINATES))
            {
                this.old_z_data = this.last_z_data;
                this.last_z_data = e;
            }
            else if ((e.Command == LxS_Cmd.ACK_X_COORDINATES) || (e.Command == LxS_Cmd.X_COORDINATES))
            {
                this.last_x_data = e;
            }
            else if (e.Command == LxS_Cmd.DATA_FIELD)
            {
                this.last_field_data = e;
            }
        }

        public void Disconnect()
        {
            try
            {
                this.lxs.Disconnect();
            }
            catch { }
            try
            {
                this.lxs.Dispose();
            }
            catch { }
            try
            {
                GC.SuppressFinalize(this.lxs);
            }
            catch { }
            this.lxs = null;
        }

        public void setLaserGate(bool state)
        {
            this.lxs.SetLaserGate(state);
        }

        public void setSensorMode(bool mode)
        {
            this.lxs.SetSensorMode((mode) ? (LxS_Mode.MEASURE) : (LxS_Mode.COMMAND));
        }

        public int getInspectionTask()
        {
            return (int)(this.lxs.GetInspectionTask());
        }

        public void setInspectionTask(int task)
        {
            this.lxs.SetInspectionTask((UInt16)task, true);
        }

        public string getVersion()
        {
            return (this.lxs.Version);
        }

        public string getData()
        {
            String ret = "NULL";
            if ((this.last_z_data != null) && (this.last_x_data != null) && (this.old_z_data != null))
            {
                StringBuilder builder = new StringBuilder();
                LxS_EventArgs x_d = this.last_x_data;
                LxS_EventArgs z_d = (x_d.ScanNumber == this.old_z_data.ScanNumber) ? (this.old_z_data) : (this.last_z_data);
                UInt16[] x = x_d.Data;
                UInt16[] z = z_d.Data;
                builder.AppendFormat("{0}:{1};", x_d.ScanNumber, x_d.EncoderValue);
                int cnt = Math.Min(x_d.Data.Length, z_d.Data.Length);
                for (int i = 0; i < cnt; i++)
                {
                    builder.AppendFormat("{0}:{1};", ((Int16)x[i]), z[i]);
                }
                ret = (builder.ToString());
            }
            return (ret);
        }

        public string getField()
        {
            String ret = "NULL";
            if (this.last_field_data != null)
            {
                StringBuilder builder = new StringBuilder();
                LxS_EventArgs f_d = this.last_field_data;
                UInt16[] f = f_d.Data;
                builder.AppendFormat("{0}:{1};", f_d.ScanNumber, f_d.EncoderValue);
                if (f != null)
                {
                    foreach (UInt16 u in f)
                    {
                        builder.Append(u);
                        builder.Append(";");
                    }
                }
                else
                {
                    builder.Append("NULL;");
                }
                ret = (builder.ToString());
            }
            return (ret);
        }

        public string getDeviceType()
        {
            return (this.lxs.GetDeviceType().ToString());
        }

        public void ethernetTrigger()
        {
            this.lxs.EthernetTrigger();
        }

        #endregion
    }
}
